/*
 *  Copyright (C) 2005  Maarten de Boer <maarten@resorama.com>
 * 
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 *  02110-1301, USA.
 * 
 */

#include <png.h>
#include <jpeglib.h>
#include "Texture.hxx"
#include "TextureLoader.hxx"

TextureLoader::TextureLoader()
{}

void TextureLoader::LoadPng( Texture& texture,const std::string& pngFilename,GLint param)
{
   LoadPngJpg(texture,pngFilename,"",param);
}

void TextureLoader::LoadJpg(Texture& texture,const std::string& jpgFilename,GLint param)
{}

void TextureLoader::LoadPngJpg( Texture& texture,const std::string& pngFilename,const std::string& jpgFilename,GLint param)
{
   texture.Bind();

   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, param);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, param);

   /* to save diskspace, i split pngs in a png and a jpg. the jpg
    * is the original png converted to jpg, and the png is the original
    * png, but with all pixels with alpha=1 set to white. obviously,
    * the resulting png+jpg is much smaller than the original png.
    *
    * this functions reads png and jpg, and combines it into a texture.
    * if jpg_file_name == NULL, or the file does not exist, it will
    * only read the png.
    *
    * note that this function has been a quick hack, also to remove
    * glpng dependency
    * 
    * todo: clean up, handle errors, check that jpg and png match, and
    * make sure png files are 8-byte RGBA
    */
   png_byte header[8];
   int number = 8;

   FILE *fp = fopen(pngFilename.c_str(), "rb");
   if (!fp) {
      throw TextureLoaderException(std::string("cannot open "+pngFilename));
   }
   fread(header, 1, number, fp);
   int is_png = !png_sig_cmp(header, 0, number);
   if (!is_png) {
      throw TextureLoaderException(pngFilename+" is not a png");
   }

   png_structp png_ptr = png_create_read_struct
                         (PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
   if (!png_ptr)
      throw TextureLoaderException("png_create_read_struct failed");

   png_infop info_ptr = png_create_info_struct(png_ptr);
   if (!info_ptr) {
      png_destroy_read_struct(&png_ptr,
                              (png_infopp) NULL, (png_infopp) NULL);
      throw TextureLoaderException("png_create_info_struct failed");
   }

   png_infop end_info = png_create_info_struct(png_ptr);
   if (!end_info) {
      png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp) NULL);
      throw TextureLoaderException("png_create_info_struct (end_info) failed");
   }
   if (setjmp(png_jmpbuf(png_ptr))) {
      png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);
      fclose(fp);
      throw TextureLoaderException("png_create_info_struct (end_info) failed");
   }
   png_init_io(png_ptr, fp);
   png_set_sig_bytes(png_ptr, number);
   png_uint_32 width, height;
   int bit_depth, color_type, interlace_type;
   png_read_info(png_ptr, info_ptr);
   png_get_IHDR(png_ptr, info_ptr, &width, &height, &bit_depth,
                &color_type, &interlace_type, int_p_NULL, int_p_NULL);

   int row_bytes = png_get_rowbytes(png_ptr, info_ptr);

   png_bytep *row_pointers =
      (png_bytep *) png_malloc(png_ptr, height * sizeof(png_bytep));
   png_bytep data = (png_bytep) png_malloc(png_ptr, height * row_bytes);
   png_bytep ptr = data;
   for (unsigned int i = 0; i < height; i++) {
      row_pointers[i] = ptr;
      ptr += row_bytes;
   }

   png_read_image(png_ptr, row_pointers);

   FILE *infile = NULL;
   if (jpgFilename!="") {
      infile = fopen(jpgFilename.c_str(), "rb");
      if (!infile)
         throw TextureLoaderException(std::string("failed to open ")+jpgFilename);

      struct jpeg_decompress_struct cinfo;
      struct jpeg_error_mgr jerr;
      cinfo.err = jpeg_std_error(&jerr);
      jpeg_create_decompress(&cinfo);
      jpeg_stdio_src(&cinfo, infile);
      jpeg_read_header(&cinfo, TRUE);
      jpeg_start_decompress(&cinfo);

      unsigned char *line =
         (unsigned char *) malloc(cinfo.output_width *
                                  cinfo.output_components);
      ptr = data;
      while (cinfo.output_scanline < cinfo.output_height) {
         jpeg_read_scanlines(&cinfo, &line, 1);
         unsigned char *jpeg_ptr = line;
         for (unsigned int i = 0; i < cinfo.output_width; i++) {
            if (ptr[0] == 255 &&
                  ptr[1] == 255 && ptr[2] == 255 && ptr[3] == 255) {
               *ptr++ = *jpeg_ptr++;
               *ptr++ = *jpeg_ptr++;
               *ptr++ = *jpeg_ptr++;
               ptr++;
            } else {
               ptr += 4;
               jpeg_ptr += 3;
            }
         }
      }
      free(line);
      jpeg_finish_decompress(&cinfo);
      jpeg_destroy_decompress(&cinfo);

      fclose(infile);
   }

   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, width, height, 0, GL_RGBA,
                GL_UNSIGNED_BYTE, data);

   png_read_end(png_ptr, NULL);
   png_free(png_ptr, row_pointers);
   png_free(png_ptr, data);
   png_destroy_read_struct(&png_ptr, &info_ptr, NULL);


   texture.rect = Rect(0,0,int(width),int(height));
}

